/*
 * This file is part of SEMP.
 *
 * SEMP is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEMP is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SEMP. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2009, 2010, 2011, 2012 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */

// SEMP.cpp : Defines the entry point for the console application.
//

/*
Thanks to Chris Thompson for his DirectShow article on Flipcode
(http://www.flipcode.com/archives/DirectShow_For_Media_Playback_In_Windows-{Part_I_Basics.shtml;Part_II_DirectShow_In_C.shtml;Part_III_Customizing_Graphs.shtml})
and the MSDN for their samples

NOTE: The Sensory Effect Media Player (SEMP) is no commercial product and should mainly be used in research
*/

/*
	KNOWN ISSUES:
	-------------
	Movies can show small artifacts at the right top corner within fullscreen mode (cause: ATI graphics cards and SampleGrabber)
	Video and audio can be out of sync because of processing power needed for calculation of average color!
	DirectShow graph is not connected between video decoder and sample grabber because of OSD (but working)
*/

#include "SEMP.h"

/**
Method for painting the window
\param hwnd application handle
*/
void OnPaint(HWND hwnd)
{
    HRESULT hr;
    PAINTSTRUCT ps;
    HDC         hdc;
    RECT        rcClient;

    GetClientRect(hwnd, &rcClient);
    hdc = BeginPaint(hwnd, &ps);

	if(g_pDirectShow != NULL && g_pDirectShow->GetVMRWindowlessControl() != NULL)
        hr = g_pDirectShow->GetVMRWindowlessControl()->RepaintVideo(hwnd, hdc);
    else  // No video image. Just paint the whole client area.
        FillRect(hdc, &rcClient, (HBRUSH)(COLOR_BTNFACE + 1));
  
    EndPaint(hwnd, &ps);
}


/**
Method for stopping the timer
*/
void StopTimer()
{
	if (m_timerID != 0)
	{
		KillTimer(g_AppWindow, m_timerID);
		m_timerID = 0;
	}
}

/**
Method for handling timer events
*/
void OnTimer()
{
	if (g_pOSD->IsOSDVisible() && g_osdType != 0)
	{
		if (g_osdType == 1)
			g_pOSD->RefreshOSD(g_AppWindow, g_pDirectShow->GetVMRWindowlessControl(), g_pDirectShow->GetMediaPosition(),
				g_pDirectShow->GetBasicAudio());
		else if (g_osdType == 2)
			g_pOSD->RefreshVotingOSD(g_AppWindow, g_pDirectShow->GetVMRWindowlessControl(), g_curVote);
	}

	REFTIME timeNow;

	if (SUCCEEDED(g_pDirectShow->GetMediaPosition()->get_CurrentPosition(&timeNow)))
    {
		timeNow = Utils::Round(timeNow, 1);
		if (g_NextRoSEEvent != timeNow)
		{
			if (g_bAutoColor || g_bStoreBMP)
			{
				// determine buffer size
				long cbBuffer = g_pDirectShow->GetAMMediaType().lSampleSize;
				unsigned char *pBuffer = new unsigned char[cbBuffer];
								
				HRESULT hr = g_pDirectShow->GetSampleGrabber()->GetCurrentBuffer(&cbBuffer, (long *)pBuffer);
				if (FAILED(hr))
				{
					// we want to ignore the startup time from directshow
					if (hr != VFW_E_WRONG_STATE)
					{
						MessageBox(NULL, "Cannot retrieve image", "Image error", MB_OK);
						StopTimer();
						PostQuitMessage(0);
					}
					/*switch (hr)
					{
					case E_INVALIDARG:
						MessageBox(NULL, "Samples are not being buffered.", "Image error", MB_OK);
						break;
					case E_OUTOFMEMORY:
						MessageBox(NULL, "The specified buffer is not large enough.", "Image error", MB_OK);
						break;
					case E_POINTER:
						MessageBox(NULL, "NULL pointer argument.", "Image error", MB_OK);
						break;
					case VFW_E_NOT_CONNECTED:
						MessageBox(NULL, "The filter is not connected.", "Image error", MB_OK);
						break;
					case VFW_E_WRONG_STATE:
						MessageBox(NULL, "The filter has not received any samples yet. To deliver a sample, run or pause the graph.", "Image error", MB_OK);
						break;
					}*/					
				}

				// provide automatic color calculation
				if (g_bAutoColor)
					CalculateColor(pBuffer, timeNow);

				// store frames 
				if (g_bStoreBMP && g_BMPPath != NULL)
				{
					if (cbBuffer > 0)
					{
						BITMAPINFOHEADER biH;
						HANDLE fh;
						BITMAPFILEHEADER bmphdr;
						DWORD nWritten;

						memset(&bmphdr, 0, sizeof(bmphdr));
						memcpy(&biH, &g_pDirectShow->GetVideoInfoHeader()->bmiHeader, sizeof(biH));

						bmphdr.bfType = ('M' << 8) | 'B';
						bmphdr.bfSize = sizeof(bmphdr) + sizeof(BITMAPINFOHEADER) + cbBuffer;
						bmphdr.bfOffBits = sizeof(bmphdr) + sizeof(BITMAPINFOHEADER);

						char name[255];
						sprintf_s(name, "%s\\image%05i.bmp", g_BMPPath, counter);

						fh = CreateFile(name, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
						WriteFile(fh, &bmphdr, sizeof(bmphdr), &nWritten, NULL);
						WriteFile(fh, &biH, sizeof(biH), &nWritten, NULL);
						WriteFile(fh, pBuffer, cbBuffer, &nWritten, NULL);
						CloseHandle(fh);
						counter++;
					}
				}

				delete pBuffer;					
			}
			
			// render sensory effects
			if (effectManager != NULL)
			{
				effectManager->PlayEffectAtTime(timeNow);

				if (effectManager->HasError())
				{
					MessageBox(NULL, effectManager->GetErrorString().c_str(), "Error", MB_OK);
					effectManager->ClearError();
					StopTimer();
					PostQuitMessage(0);
				}
			}
			
			g_NextRoSEEvent = timeNow;
		}
    }
}

/**
Method for initialising the timer
*/
void InitTimer()
{
	// Start the timer
	m_timerID = SetTimer(g_AppWindow, IDT_TIMER1, TICK_FREQ, NULL);  
}

/**
Method for color calculation
\param pBuffer containg the frame to calculate from
\param time needed for debugging information
*/
void CalculateColor(unsigned char* pBuffer, double time)
{
	long lWidth, lHeight; 
	if (g_pDirectShow != NULL && g_pDirectShow->GetVMRWindowlessControl() != NULL)
	{
		HRESULT hr = g_pDirectShow->GetVMRWindowlessControl()->GetNativeVideoSize(&lWidth, &lHeight, NULL, NULL); 
		if (SUCCEEDED(hr))
		{
			int rgb[3][3];

			if (!g_disableMMX)
				ColorCalculator::CalculateColorMMX(pBuffer, lWidth, lHeight, rgb);
			else
				ColorCalculator::CalculateColor(pBuffer, lWidth, lHeight, rgb);
		

			// format: 0=[L]R8G9B11[M]R2G1B1[R]R1G1B1
			if (g_bDebug)
			{
				char line[MAX_PATH];
				int cLine = sprintf_s(line, "%li=[L]R%iG%iB%i[M]R%iG%iB%i[R]R%iG%iB%i\n", (long)(time*g_PTS), rgb[0][0], rgb[1][0], rgb[2][0],
					rgb[0][1], rgb[1][1], rgb[2][1], rgb[0][2], rgb[1][2], rgb[2][2]);
				DWORD nWritten;
				WriteFile(colorFile, line, cLine, &nWritten, NULL);
			}

			// set light color
			// NOTE: we currently only use 3 predefined positions of lights. Maybe in a later
			// release this will change as the engine already supports all locations available
			effectManager->ColorToDevice(Classification::LOCATIONURIBASE + Classification::LOC_X_LEFT +
				Classification::LOC_EVERYWHERE + Classification::LOC_EVERYWHERE, rgb[0][0], rgb[0][1], rgb[0][2]);
			effectManager->ColorToDevice(Classification::LOCATIONURIBASE + Classification::LOC_X_CENTER +
				Classification::LOC_EVERYWHERE + Classification::LOC_EVERYWHERE, rgb[1][0], rgb[1][1], rgb[1][2]);
			effectManager->ColorToDevice(Classification::LOCATIONURIBASE + Classification::LOC_X_RIGHT +
				Classification::LOC_EVERYWHERE + Classification::LOC_EVERYWHERE, rgb[2][0], rgb[2][1], rgb[2][2]);

			effectManager->DisplayColor();
		}
	}
}

/**
Method representing the window callback
*/
LRESULT CALLBACK WindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	HRESULT hr;
	REFTIME timeNow;

    switch (uMsg)
    {
    case WM_SYSCOMMAND:
        if (wParam == SC_CLOSE)
		{
			DoCleanUp();
			PostQuitMessage(0);
		}
        break;
	case WM_GRAPHEVENT:
		g_pDirectShow->OnGraphEvent();		// handles events
		break;
	case WM_TIMER:
		OnTimer();
		break;
	case WM_KEYUP:
		switch (wParam)
		{
		case VK_ESCAPE: // ESC = exit
			DoCleanUp();
			PostQuitMessage(0);
			break;
		case 'O': // O = OSD
			if (g_osdType == 1)
				g_pOSD->PrepareAndShowOSD(g_AppWindow, g_pDirectShow->GetVMRWindowlessControl(), g_pDirectShow->GetMediaPosition(),
					g_pDirectShow->GetBasicAudio());
			break;
		case 'M': // M = mute
			if (g_pDirectShow->GetBasicAudio() != NULL)
			{
				long volume = MAX_VOLUME;
				hr = g_pDirectShow->GetBasicAudio()->get_Volume(&volume);
				if (FAILED(hr))
				{
					MessageBox(NULL, "Cannot mute!", "Error", MB_OK);
					OnPaint(g_AppWindow);
					break;
				}

				if (volume > MUTE_VOLUME)
				{
					g_lVolume = volume;
					g_pDirectShow->UpdateVolume(MUTE_VOLUME);
				}
				else
					g_pDirectShow->UpdateVolume(g_lVolume);
			}

			if (g_pOSD->IsOSDVisible() && g_osdType != 0)
			{
				if (g_osdType == 1)
					g_pOSD->RefreshOSD(g_AppWindow, g_pDirectShow->GetVMRWindowlessControl(), g_pDirectShow->GetMediaPosition(), 
						g_pDirectShow->GetBasicAudio());
				else
					g_pOSD->RefreshVotingOSD(g_AppWindow, g_pDirectShow->GetVMRWindowlessControl(), g_curVote);
			}
			break;
		case VK_ADD: // + = volume up
			if (g_pDirectShow->GetBasicAudio() != NULL)
			{
				long volume = MAX_VOLUME;
				hr = g_pDirectShow->GetBasicAudio()->get_Volume(&volume);
				if (FAILED(hr))
				{
					MessageBox(NULL, "Cannot change volume!", "Error", MB_OK);
					OnPaint(g_AppWindow);
					break;
				}

				if (volume+VOLUME_STEPPING < MAX_VOLUME)
				{
					g_lVolume = volume+VOLUME_STEPPING;
					g_pDirectShow->UpdateVolume(g_lVolume);
				}
				else 
				{
					g_lVolume = MAX_VOLUME;
					g_pDirectShow->UpdateVolume(MAX_VOLUME);
				}
			}

			if (g_pOSD->IsOSDVisible() && g_osdType != 0)
			{
				if (g_osdType == 1)
					g_pOSD->RefreshOSD(g_AppWindow, g_pDirectShow->GetVMRWindowlessControl(), g_pDirectShow->GetMediaPosition(), 
						g_pDirectShow->GetBasicAudio());
				else
					g_pOSD->RefreshVotingOSD(g_AppWindow, g_pDirectShow->GetVMRWindowlessControl(), g_curVote);
			}
			break;
		case VK_SUBTRACT: // - = volume down
			if (g_pDirectShow->GetBasicAudio() != NULL)
			{
				long volume = MAX_VOLUME;
				hr = g_pDirectShow->GetBasicAudio()->get_Volume(&volume);
				if (FAILED(hr))
				{
					MessageBox(NULL, "Cannot change volume!", "Error", MB_OK);
					OnPaint(g_AppWindow);
					break;
				}

				if (volume-VOLUME_STEPPING > MIN_VOLUME)
				{
					g_lVolume = volume-VOLUME_STEPPING;
					g_pDirectShow->UpdateVolume(g_lVolume);
				}
				else
				{
					g_lVolume = MIN_VOLUME;
					g_pDirectShow->UpdateVolume(MIN_VOLUME);
				}
			}

			if (g_pOSD->IsOSDVisible() && g_osdType != 0)
			{
				if (g_osdType == 1)
					g_pOSD->RefreshOSD(g_AppWindow, g_pDirectShow->GetVMRWindowlessControl(), g_pDirectShow->GetMediaPosition(), 
						g_pDirectShow->GetBasicAudio());
				else
					g_pOSD->RefreshVotingOSD(g_AppWindow, g_pDirectShow->GetVMRWindowlessControl(), g_curVote);
			}
			break;
		case VK_LEFT: // left arrow
			if (::GetKeyState(VK_SHIFT) < 0) // SHIFT + <- = seek back 60 seconds
			{
				LONG canSeek = OAFALSE;
				hr = g_pDirectShow->GetMediaPosition()->CanSeekBackward(&canSeek);
				if (FAILED(hr))
				{
					MessageBox(NULL, "Unable to determine back seeking capabilities!", "Error", MB_OK);
					OnPaint(g_AppWindow);
					break;
				}
				else if (canSeek == OAFALSE)
				{
					MessageBox(NULL, "Cannot seek backward!", "Error", MB_OK);
					OnPaint(g_AppWindow);
					break;
				}
				else
				{
					double curPosition = 0.0;
					hr = g_pDirectShow->GetMediaPosition()->get_CurrentPosition(&curPosition);
					if (FAILED(hr))
					{
						MessageBox(NULL, "Cannot determine current position!", "Error", MB_OK);
						OnPaint(g_AppWindow);
						break;
					}

					hr = g_pDirectShow->GetMediaPosition()->put_CurrentPosition(curPosition-60.0); // seek back 60 seconds
					if (FAILED(hr))
					{
						//MessageBox(NULL, "Cannot seek to desired position (e.g., position before beginning!)", "Error", MB_OK);
						break;
					}
				}
			}
			else if (::GetKeyState(VK_CONTROL) < 0) // CTRL + <- = reduce playrate
			{
				double rate = 1.0;
				hr = g_pDirectShow->GetMediaPosition()->get_Rate(&rate);
				if (FAILED(hr))
				{
					MessageBox(NULL, "Cannot retrieve current playback rate!", "Error", MB_OK);
					OnPaint(g_AppWindow);
					break;
				}
				else
				{
					if (rate > 0.5)
					{
						hr = g_pDirectShow->GetMediaPosition()->put_Rate(rate-0.5);
						if (FAILED(hr))
						{
							MessageBox(NULL, "Cannot set new playback rate!", "Error", MB_OK);
							OnPaint(g_AppWindow);
							break;
						}
					}
				}
			}
			else // only <- = seek back 5 seconds
			{
				LONG canSeek = OAFALSE;
				hr = g_pDirectShow->GetMediaPosition()->CanSeekBackward(&canSeek);
				if (FAILED(hr))
				{
					MessageBox(NULL, "Unable to determine back seeking capabilities!", "Error", MB_OK);
					OnPaint(g_AppWindow);
					break;
				}
				else if (canSeek == OAFALSE)
				{
					MessageBox(NULL, "Cannot seek backward!", "Error", MB_OK);
					OnPaint(g_AppWindow);
					break;
				}
				else
				{
					double curPosition = 0.0;
					hr = g_pDirectShow->GetMediaPosition()->get_CurrentPosition(&curPosition);
					if (FAILED(hr))
					{
						MessageBox(NULL, "Cannot determine current position!", "Error", MB_OK);
						OnPaint(g_AppWindow);
						break;
					}

					hr = g_pDirectShow->GetMediaPosition()->put_CurrentPosition(curPosition-5.0);
					if (FAILED(hr))
					{
						//MessageBox(NULL, "Cannot seek to desired position (e.g., position before beginning!)", "Error", MB_OK);
						break;
					}
				}
			}

			if (g_pOSD->IsOSDVisible() && g_osdType != 0)
			{
				if (g_osdType == 1)
					g_pOSD->RefreshOSD(g_AppWindow, g_pDirectShow->GetVMRWindowlessControl(), g_pDirectShow->GetMediaPosition(), 
						g_pDirectShow->GetBasicAudio());
				else
					g_pOSD->RefreshVotingOSD(g_AppWindow, g_pDirectShow->GetVMRWindowlessControl(), g_curVote);
			}
			break;
		case VK_RIGHT: // right arrow
			if (::GetKeyState(VK_SHIFT) < 0) // SHIFT + -> = seek forward 60 seconds
			{
				LONG canSeek = OAFALSE;
				hr = g_pDirectShow->GetMediaPosition()->CanSeekForward(&canSeek);
				if (FAILED(hr))
				{
					MessageBox(NULL, "Unable to determine forward seeking capabilities!", "Error", MB_OK);
					OnPaint(g_AppWindow);
					break;
				}
				else if (canSeek == OAFALSE)
				{
					MessageBox(NULL, "Cannot seek forward!", "Error", MB_OK);
					OnPaint(g_AppWindow);
					break;
				}
				else
				{
					double curPosition = 0.0;
					hr = g_pDirectShow->GetMediaPosition()->get_CurrentPosition(&curPosition);
					if (FAILED(hr))
					{
						MessageBox(NULL, "Cannot determine current position!", "Error", MB_OK);
						OnPaint(g_AppWindow);
						break;
					}

					hr = g_pDirectShow->GetMediaPosition()->put_CurrentPosition(curPosition+60.0);
					if (FAILED(hr))
					{
						//MessageBox(NULL, "Cannot seek to desired position (e.g., position after end!)", "Error", MB_OK);
						break;
					}
				}
			}
			else if (::GetKeyState(VK_CONTROL) < 0) // CTRL + -> = increase play rate
			{
				double rate = 1.0;
				hr = g_pDirectShow->GetMediaPosition()->get_Rate(&rate);
				if (FAILED(hr))
				{
					MessageBox(NULL, "Cannot retrieve current playback rate!", "Error", MB_OK);
					OnPaint(g_AppWindow);
					break;
				}
				else
				{
					if (rate < 2.0)
					{
						hr = g_pDirectShow->GetMediaPosition()->put_Rate(rate+0.5);
						if (FAILED(hr))
						{
							MessageBox(NULL, "Cannot set new playback rate!", "Error", MB_OK);
							OnPaint(g_AppWindow);
							break;
						}
					}
				}
			}
			else // only -> = seek forward 5 seconds
			{
				LONG canSeek = OAFALSE;
				hr = g_pDirectShow->GetMediaPosition()->CanSeekForward(&canSeek);
				if (FAILED(hr))
				{
					MessageBox(NULL, "Unable to determine forward seeking capabilities!", "Error", MB_OK);
					OnPaint(g_AppWindow);
					break;
				}
				else if (canSeek == OAFALSE)
				{
					MessageBox(NULL, "Cannot seek forward!", "Error", MB_OK);
					OnPaint(g_AppWindow);
					break;
				}
				else
				{
					double curPosition = 0.0;
					hr = g_pDirectShow->GetMediaPosition()->get_CurrentPosition(&curPosition);
					if (FAILED(hr))
					{
						MessageBox(NULL, "Cannot determine current position!", "Error", MB_OK);
						OnPaint(g_AppWindow);
						break;
					}

					hr = g_pDirectShow->GetMediaPosition()->put_CurrentPosition(curPosition+5.0);
					if (FAILED(hr))
					{
						//MessageBox(NULL, "Cannot seek to desired position (e.g., position after end!)", "Error", MB_OK);
						break;
					}
				}
			}

			if (g_pOSD->IsOSDVisible() && g_osdType != 0)
			{
				if (g_osdType == 1)
					g_pOSD->RefreshOSD(g_AppWindow, g_pDirectShow->GetVMRWindowlessControl(), g_pDirectShow->GetMediaPosition(), 
						g_pDirectShow->GetBasicAudio());
				else
					g_pOSD->RefreshVotingOSD(g_AppWindow, g_pDirectShow->GetVMRWindowlessControl(), g_curVote);
			}
			break;
		case VK_SPACE: // space = pausing/unpausing
			OAFilterState state;
			hr = g_pDirectShow->GetMediaControl()->GetState(INFINITE, &state);
			if (FAILED(hr))
			{
				MessageBox(NULL, "Unable to retrieve video state!", "Error", MB_OK);
				OnPaint(g_AppWindow);
				break;
			}

			if (state == State_Running)
			{
				hr = g_pDirectShow->GetMediaControl()->Pause();
				if (FAILED(hr))
				{
					MessageBox(NULL, "Unable to pause video!", "Error", MB_OK);
					OnPaint(g_AppWindow);
					break;
				}
			}
			else if (state == State_Paused)
			{
				hr = g_pDirectShow->GetMediaControl()->Run();
				if (FAILED(hr))
				{
					MessageBox(NULL, "Unable to continue video!", "Error", MB_OK);
					OnPaint(g_AppWindow);
					break;
				}
			}

			if (g_pOSD->IsOSDVisible() && g_osdType != 0)
			{
				if (g_osdType == 1)
					g_pOSD->RefreshOSD(g_AppWindow, g_pDirectShow->GetVMRWindowlessControl(), g_pDirectShow->GetMediaPosition(), 
						g_pDirectShow->GetBasicAudio());
				else
					g_pOSD->RefreshVotingOSD(g_AppWindow, g_pDirectShow->GetVMRWindowlessControl(), g_curVote);
			}
			break;
		case 'H': // H = Help
			MessageBox(NULL, g_HelpMessage.c_str(), "Help", MB_OK);
			OnPaint(g_AppWindow);
			break;
		// voting keys for performing subjective assessments
		case '1':
			if (logger != NULL)
			{
				if (SUCCEEDED(g_pDirectShow->GetMediaPosition()->get_CurrentPosition(&timeNow)))
				{
					g_curVote = 1;
					char msg[MAX_PATH];
					sprintf_s(msg, "Time: %.1f, Vote: 1", timeNow);
					logger->WriteToLog(msg);					
				}
			}
			break;
		case '2':
			if (logger != NULL)
			{
				if (SUCCEEDED(g_pDirectShow->GetMediaPosition()->get_CurrentPosition(&timeNow)))
				{
					g_curVote = 2;
					char msg[MAX_PATH];
					sprintf_s(msg, "Time: %.1f, Vote: 2", timeNow);
					logger->WriteToLog(msg);					
				}
			}
			break;
		case '3':
			if (logger != NULL)
			{
				if (SUCCEEDED(g_pDirectShow->GetMediaPosition()->get_CurrentPosition(&timeNow)))
				{
					g_curVote = 3;
					char msg[MAX_PATH];
					sprintf_s(msg, "Time: %.1f, Vote: 3", timeNow);
					logger->WriteToLog(msg);					
				}
			}
			break;
		case '4':
			if (logger != NULL)
			{
				if (SUCCEEDED(g_pDirectShow->GetMediaPosition()->get_CurrentPosition(&timeNow)))
				{
					g_curVote = 4;
					char msg[MAX_PATH];
					sprintf_s(msg, "Time: %.1f, Vote: 4", timeNow);
					logger->WriteToLog(msg);					
				}
			}
			break;
		case '5':
			if (logger != NULL)
			{
				if (SUCCEEDED(g_pDirectShow->GetMediaPosition()->get_CurrentPosition(&timeNow)))
				{
					g_curVote = 5;
					char msg[MAX_PATH];
					sprintf_s(msg, "Time: %.1f, Vote: 5", timeNow);
					logger->WriteToLog(msg);					
				}
			}
			break;
		// Enable/disable sensory effects
		case 'E':
			// it will only be deactivated until next effect in SEM occurs
			// check if enabled
			if (effectManager != NULL)
			{
				if (effectManager->AnyDeviceEnabled())
				{
					g_bAutoColor = false;
					effectManager->ActivateAllAvailableDevices(false);
				}
				else
				{
					if (g_bOldAutoColor)
						g_bAutoColor = true;
					effectManager->ActivateAllAvailableDevices(true);
				}
			}
			break;
		}
		break;
    case WM_PAINT:
		OnPaint(g_AppWindow);
        break;
	default:
		break;
    }

    return DefWindowProc(hwnd, uMsg, wParam, lParam);
}

/**
Method for opening a multimedia file
*/
void OpenFile()
{
	if (g_pDirectShow != NULL)
	{
		g_pDirectShow->CleanUpDirectShow();    // release previous graph
		delete g_pDirectShow;
		g_pDirectShow = NULL;
	}

	g_pDirectShow = new DirectShow(g_pOSD, g_bDebug);

	if (g_pDirectShow->InitDirectShow(g_AppWindow, g_bAutoColor, g_bStoreBMP, g_disableMMX) != 0)
	{
		MessageBox(NULL, "Could not initialize DirectShow!", "ERROR", MB_OK);
		PostQuitMessage(0);
	}
	else
	{
		if (g_MediaFile == NULL)
		{
			OPENFILENAME    ofn;    // use common 'Open' control
			char            file[MAX_PATH];     // filename with path
			char            filetitle[MAX_PATH];    // thrown in to show file in title bar

			memset(file, 0, MAX_PATH);
			memset(&ofn, 0, sizeof(OPENFILENAME));
			ofn.lStructSize = sizeof(OPENFILENAME);
			ofn.hwndOwner = NULL;
			ofn.lpstrFile = file;
			ofn.nMaxFile = MAX_PATH-1;
			ofn.lpstrFileTitle = filetitle;
			ofn.nMaxFileTitle = MAX_PATH-1;
			ofn.lpstrFilter = FILE_FILTER_TEXT;
			ofn.Flags = OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST;
			if (GetOpenFileName(&ofn))
			{
				g_MediaFile = file;
			}
			else
			{
				PostQuitMessage(0);
				return;
			}
		}

		if (g_pDirectShow->CreateGraph(g_MediaFile, g_ExecuteDir, g_lVolume, effectManager) != 0)
		{
			MessageBox(NULL, "Could not create a filter graph for this file!",
				"ERROR", MB_OK);
			PostQuitMessage(0);
		}
		else
		{
			ShowCursor(false);
			g_pDirectShow->GetMediaControl()->Run();
			InitTimer();
			if (g_osdType == 2)
				g_pOSD->PrepareAndShowVotingOSD(g_AppWindow, g_pDirectShow->GetVMRWindowlessControl());
		}
	}
}

/**
Method for enumerating available monitors
*/
BOOL CALLBACK EnumMonitorsProc(HMONITOR hMonitor, HDC hdc, LPRECT lpRMonitor, LPARAM dwData)
{
	if (g_desiredMonitor > 0)
		g_desiredMonitor--; // if 0 we have found our monitor
	else if (g_desiredMonitor == 0)
	{
	   MONITORINFOEX mix;
	   memset(&mix, 0, sizeof(MONITORINFOEX));
	   mix.cbSize = sizeof(MONITORINFOEX);

	   if (GetMonitorInfo(hMonitor, &mix))
	   {		   
		   g_rectDesiredMonitor = mix.rcMonitor;
	   }
	   g_desiredMonitor--;
   }

   return true;
}

/**
Method for initialising the applicaiton
\return -1 if error else 0
*/
int InitApp()
{
    WNDCLASSEX wc;

    CoInitialize(NULL);

	if (GetSystemMetrics(SM_CMONITORS) >= g_desiredMonitor)
	{
		//check number of monitors    
		EnumDisplayMonitors(NULL, NULL, EnumMonitorsProc, NULL);
	}

    memset(&wc,0, sizeof(wc));
    wc.cbSize = sizeof(wc);
    wc.lpfnWndProc = WindowProc;
    wc.hInstance = g_AppInstance;
    wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
    wc.lpszClassName = "SEMP Class";
    RegisterClassEx(&wc);

	int cx = 0;
	int cy = 0;
	int x = 0;
	int y = 0;

	if (g_desiredMonitor == 0)
	{
		// width
		cx = GetSystemMetrics(SM_CXSCREEN);
		// height
		cy = GetSystemMetrics(SM_CYSCREEN);
	}
	else
	{
		x = g_rectDesiredMonitor.left;
		y = g_rectDesiredMonitor.top;
		cx = g_rectDesiredMonitor.right - g_rectDesiredMonitor.left;
		cy = g_rectDesiredMonitor.bottom - g_rectDesiredMonitor.top;
	}

    g_AppWindow = CreateWindowEx(0, "SEMP Class",
                                g_WindowTitle,
                                WS_EX_TOPMOST | WS_POPUP,
                                x,
                                y,
                                cx,
                                cy,
                                NULL,
                                NULL,
                                g_AppInstance,
                                NULL);
    if (g_AppWindow == NULL)
        return -1;

	ShowWindow(g_AppWindow, SW_SHOW);

    // save directory for log file
    GetCurrentDirectory(MAX_PATH, g_ExecuteDir);

    return 0;
}

/**
Method for initialising the Sensory Effect Metadata (SEM) description
\param semFilePath path to the SEM description
\return false if an error occured else true
*/
bool InitSEM(char *semFilePath)
{
	if (semFilePath != NULL && strcmp(semFilePath, "") != 0)
    {
		Libxml *xml = new Libxml();
		effectManager = new EffectManager(g_bDisableFateTime, xml->Parse(semFilePath), g_ScentConfigFile);
		g_bAutoColor = xml->IsAutoColorActivated();
		g_bOldAutoColor = g_bAutoColor;
		g_PTS = xml->GetTimeScale();
		delete xml;
    }
	else if (g_bAutoColor)
		effectManager = new EffectManager(g_bDisableFateTime, g_ScentConfigFile);

	if (effectManager != NULL && effectManager->HasError())
	{
		MessageBox(NULL, effectManager->GetErrorString().c_str(), "ERROR", MB_OK);
		
		if (effectManager != NULL)
		{
			delete effectManager;
			effectManager = NULL;
		}

		g_bAutoColor = false;
		g_bOldAutoColor = false;

		return false;
	}

	if (g_bAutoColor)
	{
		effectManager->ActivateAutoColor(true);	
	}

	// activate all devices
	effectManager->InitialActivateAllAvailableDevices(true);

	if (g_bDebug)
	{
		char colorFileName[MAX_PATH];
		sprintf_s(colorFileName, "%s\\colorFile.txt", g_ExecuteDir);
		colorFile = CreateFile(colorFileName, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);

		char header[11];
		int cHeader = sprintf_s(header, "PTS=%i\n", g_PTS);
		DWORD nWritten;
		WriteFile(colorFile, header, cHeader, &nWritten, NULL);
	}

	return true;
}

/**
Method for initialising the help message
*/
void InitHelpMessage()
{
	g_HelpMessage += "(c) 2009-2012 Waltl Markus (Alpen-Adria-Universitt Klagenfurt)\n";
	g_HelpMessage += "-----------------------------------------------------------------------------\n\n";
	g_HelpMessage += "Player controls:\n";
	g_HelpMessage += "----------------\n";
	g_HelpMessage += "+\t\t: Volume up\n";
	g_HelpMessage += "-\t\t: Volume down\n";
	g_HelpMessage += "M\t\t: Mute\n";
	g_HelpMessage += "ESC\t\t: Exit player\n";
	g_HelpMessage += "->\t\t: Five seconds forward\n";
	g_HelpMessage += "<-\t\t: Five seconds backward\n";
	g_HelpMessage += "SHIFT+->\t: 60 seconds forward\n";
	g_HelpMessage += "SHIFT+<-\t: 60 seconds backward\n";
	g_HelpMessage += "CTRL+->\t\t: Increase playback rate\n";
	g_HelpMessage += "CTRL+<-\t\t: Decrease playback rate\n";
	g_HelpMessage += "SPACE\t\t: Pause video (current sensory effects are still playing!)\n";
	g_HelpMessage += "O\t\t: On-Screen-Display\n";
	g_HelpMessage += "1 - 5\t\t: Voting (repeatable) (1: very annoying; 5: big enhancement)\n";
#if __USEAMBX__ || __USEVORTEX__
	g_HelpMessage += "E\t\t: Enable/Disable sensory effects\n"; //ambx + vortex
#endif
	g_HelpMessage += "H\t\t: Help window\n";
	g_HelpMessage += "\n\n";
	g_HelpMessage += "Player parameters:\n";
	g_HelpMessage += "------------------\n";
	g_HelpMessage += "-M <media>\t: Video file to play\n";
	g_HelpMessage += "-V 0-100\t\t: Initial volume in percent\n";
	g_HelpMessage += "-D\t\t: Debug, log of directshow graph, graph file and other logs will be generated\n";
#if __USEAMBX__ || __USEVORTEX__
	g_HelpMessage += "-S <sem>\t: Sensory effect metadata description\n"; //ambx or vortex
#endif
#if __USEAMBX__
	g_HelpMessage += "-AC\t\t: Activate auto color calculation\n"; //ambx
	g_HelpMessage += "-DFT\t\t: Disable default fade time (30ms) for amBX effects\n"; //ambx
#endif
#if __USEVORTEX__
	g_HelpMessage += "-SC\t\t: File representing the scent config (if not default will be used)\n";
#endif
	g_HelpMessage += "-B <path>\t: Write bitmaps retrieved by ISampleGrabber to the given path\n";
	g_HelpMessage += "-NOMMX\t\t: Disables color calculation with MMX-SIMD support\n";
	g_HelpMessage += "-L <log-file>\t: File for storing voting results of given media\n";
	g_HelpMessage += "-Monitor x\t: Display application on monitor x\n";
	g_HelpMessage += "-OSD 0-2\t\t: Disable/Enable OSD (0 = disable, 1 = normal (default), 2 = voting)\n";
	g_HelpMessage += "-H\t\t: Only show help window and exit application";
}

/**
Method for cleaning the application on shutdown
*/
void DoCleanUp()
{
	StopTimer();

	if (effectManager != NULL)
	{
		delete effectManager;
		effectManager = NULL;
	}
	g_pDirectShow->CleanUpDirectShow();

	CoUninitialize();
	ShowCursor(true);

	if (g_pOSD != NULL)
	{
		delete g_pOSD;
		g_pOSD = NULL;
	}

	if (logger != NULL) 
	{
		delete logger;
		logger = NULL;
	}
}

/**
Application Entry Point
*/
int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
    g_AppInstance = hInstance;

	InitHelpMessage();

	char* preFile = NULL;

	if (__argc > 1 && __argv != NULL)
	{
		for (int i = 1; i < __argc; i++)
		{
			if (_strcmpi(__argv[i], "-V") == 0)
			{
				if (i+1 < __argc)
				{
					int percentVolume = atoi(__argv[++i]);
					g_lVolume = Utils::PercentToVolume(percentVolume);
				}
			}
			else if (_strcmpi(__argv[i], "-M") == 0)
			{
				if (i+1 < __argc)
					g_MediaFile = __argv[++i];
			}
			else if (_strcmpi(__argv[i], "-D") == 0)
				g_bDebug = true;
			else if (_strcmpi(__argv[i], "-H") == 0)
			{
				MessageBox(NULL, g_HelpMessage.c_str(), "Help", MB_OK);
				return 0;
			}
			else if (_strcmpi(__argv[i], "-L") == 0)
			{
				if (i+1 < __argc)
					g_LogFile = __argv[++i];
			}
			else if (_strcmpi(__argv[i], "-OSD") == 0)
			{
				if (i+1 < __argc)
					g_osdType = atoi(__argv[++i]);
			}
#if __USEAMBX__ || __USEVORTEX__
			else if (_strcmpi(__argv[i], "-S") == 0) //ambx + vortex
			{
				if (i+1 < __argc)
					g_SEMFile = __argv[++i];
			}
#endif
#if __USEAMBX__
			else if (_strcmpi(__argv[i], "-AC") == 0) //ambx
			{
				g_bAutoColor = true;
				g_bOldAutoColor = true;
			}
			else if (_strcmpi(__argv[i], "-DFT") == 0)  //ambx
				g_bDisableFateTime = true;
#endif
			else if (_strcmpi(__argv[i], "-NOMMX") == 0)
				g_disableMMX = true;	
			else if (_strcmpi(__argv[i], "-B") == 0)
			{
				if (i+1 < __argc)
				{
					g_BMPPath = __argv[++i];
					if (!Utils::IsPathValid(g_BMPPath))
						MessageBox(NULL, "Path for storing BMP files is not valid! Not storing BMPs!", "Warning", MB_OK);
					else
						g_bStoreBMP = true;
				}
			}
			else if (_strcmpi(__argv[i], "-Monitor") == 0)
			{
				if (i+1 < __argc)
					g_desiredMonitor = atoi(__argv[++i]);
			}
#if __USEVORTEX__
			else if (_strcmpi(__argv[i], "-SC") == 0) //vortex
			{
				if (i+1 < __argc)
					g_ScentConfigFile = __argv[++i];
			}
#endif
		}
	}

	if (InitApp() != 0)
	{
		DoCleanUp();
        return -1;
	}

	if (g_LogFile != NULL)
	{
		logger = new Logger(g_LogFile);
		if (logger == NULL)
		{
			MessageBox(NULL, "Unable to initialize Logger!", "ERROR", MB_OK);
			DoCleanUp();
			return -1;
		}
	}

	if (g_SEMFile != NULL || g_bAutoColor) //ambx
	{
		if (InitSEM(g_SEMFile) != true)
		{
			if (effectManager != NULL)
			{
				delete effectManager;
				effectManager = NULL;
			}
		}
	}

	OpenFile();

	MSG msg;
    while (GetMessage(&msg, NULL, 0, 0) != 0)
    {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }

	DoCleanUp();
	return 0;
}
