/*
 * This file is part of SEMP.
 *
 * SEMP is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEMP is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SEMP. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2009, 2010, 2011, 2012 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */

#ifndef SEMP_H
#define SEMP_H

// add needed libraries
#pragma comment(lib,"Strmiids.lib") // included in Windows SDK
#pragma comment(lib,"3rdParties/libs/strmbase.lib") // will be generated by BaseClasses project
#pragma comment(lib,"3rdParties/libs/libxml2.lib")

#include <windows.h>
#include "stdafx.h"

#include <string>
#include "logging/Logger.h"
#include <EffectManager.h>
#include <Libxml.h>
#include "main/Utils.h"
#include "main/OSD.h"
#include "main/DirectShow.h"
#include "main/ColorCalculator.h"



// ###########################
// ## DEFINES				##
// ###########################
// needs to be updated to support more file formats (depending on available DirectShow filters)
#define FILE_FILTER_TEXT \
    TEXT("Media Files\0*.avi; *.divx; *.mpg; *.mpeg; *.mpe; *.m1v; *.m2v; *.mpv2; *.mp2v; *.m2p; *.vob; *.evo; *.mod; \
	*.ts; *.m2ts; *.m2t; *.mts; *.pva; *.tp; *.tpr; *.mp4; *.m4v; *.mp4v; *.mpv4; *.hdmov; *.mov; *.qt; \
	*.3gp; *.3gpp; *.3g2; *.3gp2; *.mkv; *.webm; *.ogm; *.ogv; *.flv; *.wmv; *.amv; *.mp3; *.ogg; *.oga; \
	*.mka; *.m4a; *.aac; *.flac; *.wv; *.mpc; *.ofr; *.ofs; *.ape; *.apl; *.alac; *.amr; *.ac4; *.dts; \
	*.it; *.mo3; *.mtm; *.s3m; *.umx; *.xm\0All Files\0*.*\0\0")

#define WINDOWTITLE	"SEMP v0.12"

long g_PTS = 90000; // needed for color calculation (default)

// ###########################
// ## VARIABLES				##
// ###########################
HINSTANCE   g_AppInstance;
HWND    g_AppWindow;
char    g_ExecuteDir[MAX_PATH];     // directory of execution
char    g_WindowTitle[] = WINDOWTITLE;              // text for title bar
char*	g_MediaFile = NULL;
long	g_lVolume = -2500;	// default volume: 50%
bool	g_bDebug = false;
std::string g_HelpMessage;
HANDLE  debugFile = NULL;
const UINT_PTR	IDT_TIMER1 = 1;		// Timer ID
const UINT		TICK_FREQ = 100;	// Timer frequency in msec
UINT_PTR	m_timerID = 0;
char*	g_LogFile = NULL;
int	g_desiredMonitor = 0;
RECT g_rectDesiredMonitor;

int		g_osdType = 1;	// 0 = OSD disabled, 1 = normal OSD, 2 = voting OSD
static Logger *logger = NULL;
int		g_curVote = 0;
DirectShow* g_pDirectShow = NULL;

OSD* g_pOSD = new OSD();

// Variables for Sensory Effects
static EffectManager *effectManager = NULL;
double		g_NextRoSEEvent = -1.0;
char*	g_SEMFile = NULL;
int counter = 0;
HANDLE	colorFile;
bool g_bAutoColor = false;
bool g_bOldAutoColor = false;
char*	g_BMPPath = NULL;
bool g_bStoreBMP = false;
bool g_bDisableFateTime = false;
bool g_disableMMX = false;
std::string g_ScentConfigFile = "";



// ###########################
// ## FUNCTIONS				##
// ###########################
bool InitSEM(char *semFilePath);
void CalculateColor(unsigned char* pBuffer, double time);
void OnPaint(HWND hwnd);
LRESULT CALLBACK WindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
void OpenFile();
int InitApp();
void InitHelpMessage();
void DoCleanUp();
void StopTimer();
void OnTimer();
void InitTimer();

#endif