/*
 * This file is part of SEMP.
 *
 * SEMP is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEMP is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SEMP. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2009, 2010, 2011, 2012 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */

#include "Utils.h"

/**
Method for rounding a double to a specific precision
\param number to round
\param precision to round to
\return rounded number
*/
double Utils::Round(double number, int precision)
{
	number *= pow(10.0, precision);
	if (number >= 0)
		number = floor(number + 0.5);
	else
		number = ceil(number - 0.5);

	number /= pow(10.0, precision);

	return number;
}

/**
Method for simple rounding without precision
\param number to round
\return rounded number
*/
double Utils::Round(double value)
{
	return floor(value + 0.5);
}

/**
Method for calculating the volume to percent
\param volume to transform
\return percentage
*/
int Utils::VolumeToPercent(long volume)
{
	return (volume - MIN_VOLUME) / ((MAX_VOLUME - MIN_VOLUME) / 100);
}

/**
Method for calculating the percent to volume
\param percentage to transform
\return volume
*/
long Utils::PercentToVolume(int percent)
{
	return MIN_VOLUME + (percent * ((MAX_VOLUME - MIN_VOLUME) / 100));
}

/**
Method for checking if a string is a hex value
\param hexValue string to be checked
\return true if hex value else false
*/
bool Utils::IsHex(std::string hexValue)
{
	if (hexValue.length() == 6)
	{
		char cpHex [8];
		strcpy_s(cpHex, sizeof(cpHex), hexValue.c_str());
		char* nextToken = NULL;
		char* token = strtok_s(cpHex, "0123456789ABCDEFabcdef", &nextToken);
		return (token == NULL);
	}
	else
		return false;
}

/**
Method for clamping a value to a lower or upper bound
\param value to clamp
\param lowerBound lower bound to check
\param upperBound upper bound to check
\return clamped value
*/
int Utils::Clamp(int value, int lowerBound, int upperBound)
{
	if(value > upperBound || value < lowerBound)
	{
		if(value > upperBound) 
			value = upperBound;
		else
			value = lowerBound;
	}
	
	return value;
}

/**
Method for splitting a string
\param str string to split
\param delim delimiter to use
\return vector with splitted string

method taken from http://www.computing.net/answers/programming/string-split-for-c/13456.html
*/
std::vector<std::string> Utils::StringSplit(std::string str, std::string delim)
{
	std::vector<std::string> results;
	int cutAt;
	while( (cutAt = str.find_first_of(delim)) != str.npos )
	{
		if(cutAt > 0)
		{
			results.push_back(str.substr(0,cutAt));
		}
		str = str.substr(cutAt+1);
	}
	
	if(str.length() > 0)
	{
		results.push_back(str);
	}

	return results;
}

/**
Method for checking if a path is valid
\param path to be checked
\return true if path is valid else false
*/
bool Utils::IsPathValid(char *path)
{
	bool retVal = false;
	if (_access(path, 0) == 0)
	{
		struct stat status;
		stat(path, &status);

		if (status.st_mode & S_IFDIR)
		{
			retVal = true;
		}
	}

	return retVal;
}
