/*
 * This file is part of SEMP.
 *
 * SEMP is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEMP is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SEMP. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2009, 2010, 2011, 2012 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */

#ifndef UTILS_H
#define UTILS_H

#include "../stdafx.h"
#include <string>
#include <vector>
#include <io.h>
#include <sys/types.h>
#include <sys/stat.h>

// defines for specifying the volume
#define MIN_VOLUME	-5000
#define MAX_VOLUME	0
#define MUTE_VOLUME -10000
#define VOLUME_STEPPING	500

// convenient macro for releasing interfaces
#define HELPER_RELEASE(x)   if (x != NULL) \
                            { \
                                x->Release(); \
                                x = NULL; \
                            }

// define for millisecond
#define HUNDRET_MSEC 1000000.0
#define WM_GRAPHEVENT	WM_USER		// define a custom window message for graph events

class Utils
{
public:
	static long PercentToVolume(int percent);
	static int VolumeToPercent(long volume);
	static double Round(double number, int precision);
	static double Round(double value);
	static bool IsHex(std::string hexValue);
	static std::vector<std::string> StringSplit(std::string str, std::string delim);
	static int Clamp(int value, int lowerBound, int upperBound);
	static bool IsPathValid(char *path);
};

#endif