/*
 * This file is part of SEMP.
 *
 * SEMP is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEMP is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SEMP. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2009, 2010, 2011, 2012 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */

#ifndef EFFECTMANAGER_H
#define EFFECTMANAGER_H

#include "../stdafx.h"

#if __USEAMBX__
	#pragma comment(lib,"3rdParties/libs/ambxrt.lib")
#endif

#if __USEVORTEX__
	#pragma comment(lib,"3rdParties/libs/FTD2XX.lib")
#endif

#include <vector>
#include <string>
#include "Element.h"
#include "Classification.h"

#if __USEAMBX__
	#include "../3rdParties/ambx/IamBX.h"
	#include "../3rdParties/ambx/IamBX_Light.h"
	#include "../3rdParties/ambx/IamBX_Fan.h"
	#include "../3rdParties/ambx/IamBX_Rumble.h"
	#include "AmBXLocationMap.h"
	#include "LightContainer.h"
	#include "RumbleContainer.h"
	#include "FanContainer.h"
	#include "Fan.h"
	#include "Rumble.h"
	#include "Light.h"
#endif

#if __USEVORTEX__
	#include "../3rdParties/ftd2xx/FTD2XX.h"
	#include "ScentContainer.h"
	#include "Scent.h"
#endif

#include "../main/Utils.h"
#include "../logging/Logger.h"

class EffectManager
{
private:
	std::vector<Element*> m_Elements;
	int error;

#if __USEAMBX__
	AmBXLocationMap amBXLocationMap;
	IamBX* pIamBX;
	LightContainer *lightContainer;
	FanContainer *fanContainer;
	RumbleContainer *rumbleContainer;
	void FadeVibration(double time, Rumble *rumble);
	void FadeWind(double time, Fan* fan);
	// FadeLight not needed due to the available function setFadeTime of amBX
#endif

#if __USEVORTEX__
	int CalculateVortexActiveCode(int *activeFans);
	void InitializeVortexActive(std::string scentConfiguration = "");
	ScentContainer *scentContainer;
	// FadeScent not needed as the current used hardware does not support intensity values
	bool OpenScentDevice(int device);
	bool CloseScentDevice(ScentContainer *scentCont);
#endif

	void HandleWind(Element e);
	void HandleVibration(Element e);
	void HandleLight(Element e);
	void HandleScent(Element e);
	
	void StopAllFans();
	void StopAllRumblers();
	//void StopAllLights();

	HANDLE effectMutex;
	HANDLE globalLock;
	bool devChanged;

	// average update intervall calculation vars
	double i_stime;
	double i_mtime;
	double add_tspan;

	double videoLength;
	bool lightAutoColor;
	bool oldLightAutoColor; // needed for disabling and enabling effects
	Classification classification;
	bool bDisableFateTime;
	float CalculatePercent(int value);
	static bool ElementCompare(const Element* element1, const Element* element2);
	void Initialisation(std::string scentConfiguration);
	bool zeroTime;
	bool IsInIntensityRange(double value, std::vector<std::string> range);
	void DetectAttachedDevices();

public:
	void DisplayColor();
	EffectManager(bool disableFateTime, std::vector<Element*> elements, std::string scentConfiguration = "");
	EffectManager(bool disableFateTime, std::string scentConfiguration = "");
	~EffectManager();

	void ActivateAutoColor(bool activate);
	bool PlayEffect(Element e);
	bool PlayEffectAtTime(double time);
	void SetVideoLength(double length);
	double GetVideoLength();
	void ColorToDevice(std::string location, int r, int g, int b);

	std::string GetErrorString();
	bool HasError();
	void ClearError();
	int GetItems();
	void AddEffect(Element *e);
	void AddEffects(std::vector<Element*> elements);
	bool AnyDeviceEnabled();
	void ActivateAllRumblers(bool activate);
	void ActivateAllLights(bool activate);
	void ActivateAllFans(bool activate);
	void ActivateAllAvailableDevices(bool activate);
	void InitialActivateAllAvailableDevices(bool activate);

#if __USEVORTEX__
	bool ActivateAllScentDeviceFans(ScentContainer *scentCont, bool activate, bool backup = false); // backup parameter used for storing previous settings
	bool ActivateScentDeviceFan(ScentContainer *scentCont, std::string scent, bool activate);
	void ActivateAllScents(ScentContainer *scentCont, bool activate);
#endif

#if __USEAMBX__
	void Update_amBX();
#endif
};

#endif

