/*
 * This file is part of SEMP.
 *
 * SEMP is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEMP is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SEMP. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2009, 2010, 2011, 2012 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */

#include "Fan.h"

#if __USEAMBX__
/**
Constructor
\param fan with the amBX fan
*/
Fan::Fan(IamBX_Fan* fan)
{
	bFanFade = false;
	fanFadeTime = 0.0;
	fanEffectDifference = -1.0;
	fanOldIntensityValue = 0.0;
	fanDuration = -1.0;
	fanIntensity = -1.0;
	fanIntermediateIntensity = 0.0;
	fanPTS = 0.0;
	fanActive = false;
	m_fan = fan;
}

/**
Destructor
*/
Fan::~Fan()
{
}

/**
Method for resetting the fan
*/
void Fan::ResetFan()
{
	bFanFade = false;
	fanFadeTime = 0.0;
	fanEffectDifference = -1.0;
	fanOldIntensityValue = 0.0;
	fanDuration = -1.0;
	fanIntensity = -1.0;
	fanIntermediateIntensity = 0.0;
	fanPTS = 0.0;
	fanActive = false;
}

/**
Method for setting the intensity of the fan
\param intensity with the intensity
\return amBX_RESULT with the error code or ok
*/
amBX_RESULT Fan::SetIntensityToDevice(float intensity)
{
	if(fanActive)
		return m_fan->setIntensity(m_fan,intensity/100.0f);
	else
		return amBX_ERROR;
}

/**
Method for retrieving the intensity of the fan
\return float with the intensity
*/
float Fan::GetIntensityFromDevice()
{
	amBX_float _intensity;
	amBX_RESULT rc = m_fan->getIntensity(m_fan,&_intensity);
	if(rc == amBX_OK)
	{
		return _intensity*100.0f;
	}
	else
	{
		return -1.0f;
	}
}

/**
Method for determining if fan is active
\return true if active else false
*/
bool Fan::IsFanActive()
{
	return fanActive;
}

/**
Method for activating the fan
\param activate true if it should be activated else false
*/
void Fan::ActivateFan(bool activate)
{
	fanActive = activate;
}

/**
Getter for the timestamp
\return timestamp
*/
double Fan::GetPTS() 
{ 
	return fanPTS; 
}

/**
Setter for the timestamp
\param time with the timestamp
*/
void Fan::SetPTS(double time) 
{ 
	fanPTS = time; 
}

/**
Getter for the intermediate intensity
used for fading
\return intermediate intensity
*/
double Fan::GetIntermediateIntensity()
{
	return fanIntermediateIntensity;
}

/**
Setter for the intermediate intensity
\param intensity
*/
void Fan::SetIntermediateIntensity(double intensity)
{
	fanIntermediateIntensity = intensity;
}

/**
Method for determining if fading is active
\return true if active else false
*/
bool Fan::IsFadeActive()
{
	return bFanFade;
}

/**
Method for activating fading
\param activate true for activating fading else false
*/
void Fan::ActivateFade(bool activate)
{
	bFanFade = activate;
}

/**
Getter for the effect difference
used for fading
\return effect difference
*/
double Fan::GetEffectDifference()
{
	return fanEffectDifference;
}

/**
Getter for the old intensity value
\return old intensity value
*/
double Fan::GetOldIntensityValue()
{
	return fanOldIntensityValue;
}

/**
Getter for the effect duration
\return duration
*/
double Fan::GetDuration()
{
	return fanDuration;
}

/**
Getter for the intensity
\return intensity
*/
double Fan::GetIntensity()
{
	return fanIntensity;
}

/**
Getter for the fading time
\return fading time
*/
double Fan::GetFadeTime()
{
	return fanFadeTime;
}

/**
Setter for the intensity
\param intensity
*/
void Fan::SetIntensity(double intensity)
{
	fanIntensity = intensity;
}

/**
Setter for the duration
\param duration
*/
void Fan::SetDuration(double duration)
{
	fanDuration = duration;
}

/**
Setter for the old intensity
\param old intensity value
*/
void Fan::SetOldIntensityValue(double oldIntensity)
{
	fanOldIntensityValue = oldIntensity;
}

/**
Setter for the effect difference
\param effect difference
*/
void Fan::SetEffectDifference(double difference)
{
	fanEffectDifference = difference;
}

/**
Setter for the fade time
\param fade time
*/
void Fan::SetFadeTime(double fadeCount)
{
	fanFadeTime = fadeCount;
}

/**
Getter for retrieving the fan
\return amBX fan
*/
IamBX_Fan *Fan::GetFan()
{
	return m_fan;
}
#endif
