/*
 * This file is part of SEMP.
 *
 * SEMP is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEMP is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SEMP. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2009, 2010, 2011, 2012 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */

#include "FanContainer.h"

#if __USEAMBX__
/**
Constructor
*/
FanContainer::FanContainer()
{
}

/**
Destructor
*/
FanContainer::~FanContainer()
{
}

/**
Method for adding a fan to a location
\param location for the fan to be added to
\param fan with the fan to be added
*/
void FanContainer::AddFan(amBX_loc location, Fan* fan)
{
	fanMap.insert(std::make_pair(location, fan));	  
}

/**
Method for retrieving a fan
\param location from which the fan should be retrieved
\return fan
*/
Fan* FanContainer::GetFan(amBX_loc location)
{
	std::map<amBX_loc, Fan*>::iterator iter = fanMap.find(location);
	if(iter == fanMap.end())
		return NULL;
	else
		return iter->second;
}

/**
Method for retrieving all fans
\return vector with all fans
*/
std::vector<Fan *> FanContainer::GetAllFans()
{
	std::vector<Fan *> retVal;

	for( std::map<amBX_loc, Fan*>::iterator it = fanMap.begin(); it != fanMap.end(); ++it ) {
        retVal.push_back(it->second);
    }

	return retVal;
}
#endif
