/*
 * This file is part of SEMP.
 *
 * SEMP is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEMP is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SEMP. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2009, 2010, 2011, 2012 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */
#include "LightContainer.h"

#if __USEAMBX__
/**
Constructor
*/
LightContainer::LightContainer()
{
}

/**
Destructor
*/
LightContainer::~LightContainer()
{
}

/**
Method for adding a light to a location
\param location for the light to be added to
\param light with the light to be added
*/
void LightContainer::AddLight(amBX_loc location, Light* light)
{
	lightMap.insert(std::make_pair(location, light));	  
}

/**
Method for retrieving a light
\param location from which the light should be retrieved
\return light
*/
Light* LightContainer::GetLight(amBX_loc location)
{
	std::map<amBX_loc, Light*>::iterator iter = lightMap.find(location);
	if(iter == lightMap.end())
		return NULL;
	else
		return iter->second;
}

/**
Method for retrieving all lights
\return vector with all lights
*/
std::vector<Light *> LightContainer::GetAllLights()
{
	std::vector<Light *> retVal;

	for( std::map<amBX_loc, Light*>::iterator it = lightMap.begin(); it != lightMap.end(); ++it ) {
        retVal.push_back(it->second);
    }

	return retVal;
}
#endif
