/*
 * This file is part of SEMP.
 *
 * SEMP is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEMP is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SEMP. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2009, 2010, 2011, 2012 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */

#include "Rumble.h"

#if __USEAMBX__
/**
Constructor
\param rumble with the amBX rumble
*/
Rumble::Rumble(IamBX_Rumble* rumble)
{
	bRumbleFade = false;
	rumbleFadeTime = -1.0;
	rumbleEffectDifference = -1.0;
	rumbleOldIntensityValue = 0.0;
	rumbleDuration = -1.0;
	rumbleIntensity = -1.0;
	rumblePTS = 0.0;
	rumbleIntermediateIntensity = 0.0;
	rumbleActive = false;
	m_rumble = rumble;
}

/**
Destructor
*/
Rumble::~Rumble()
{
}

/**
Method for resetting the rumble
*/
void Rumble::ResetRumble()
{
	bRumbleFade = false;
	rumbleFadeTime = -1.0;
	rumbleEffectDifference = -1.0;
	rumbleOldIntensityValue = 0.0;
	rumbleDuration = -1.0;
	rumbleIntensity = -1.0;
	rumblePTS = 0.0;
	rumbleIntermediateIntensity = 0.0;
}

/**
Method for setting the intensity of the rumble
\param intensity with the intensity
\return amBX_RESULT with the error code or ok
*/
amBX_RESULT Rumble::SetIntensityToDevice(float intensity)
{
	if(rumbleActive)
		return m_rumble->setRumble(m_rumble,"amBX_road", 5.0f, intensity/100.0f);
	else
		return amBX_ERROR;
}

/**
Method for retrieving the intensity of the rumbe
\return float with the intensity
*/
float Rumble::GetIntensityFromDevice()
{
	amBX_float _intensity, _speed;
	amBX_u32 _length;
	amBX_u32 _typeLength = 255;
	amBX_char _type[255];
	amBX_RESULT rc = m_rumble->getRumble(m_rumble, _typeLength, _type, &_length, &_speed, &_intensity);
	if(rc == amBX_OK)
	{
		return _intensity*100.0f;
	}else{
		return -1.0f;
	}
}

/**
Method for determining if rumble is active
\return true if active else false
*/
bool Rumble::IsRumbleActive()
{
	return rumbleActive;
}

/**
Method for activating the rumble
\param activate true if it should be activated else false
*/
void Rumble::ActivateRumble(bool activate)
{
	rumbleActive = activate;
}

/**
Getter for the timestamp
\return timestamp
*/
double Rumble::GetPTS()
{ 
	return rumblePTS; 
}

/**
Setter for the timestamp
\param time with the timestamp
*/
void Rumble::SetPTS(double time)
{ 
	rumblePTS = time;
}

/**
Getter for the intermediate intensity
used for fading
\return intermediate intensity
*/
double Rumble::GetIntermediateIntensity()
{
	return rumbleIntermediateIntensity;
}

/**
Setter for the intermediate intensity
\param intensity
*/
void Rumble::SetIntermediateIntensity(double intensity)
{
	rumbleIntermediateIntensity = intensity;
}

/**
Method for determining if fading is active
\return true if active else false
*/
bool Rumble::IsFadeActive()
{
	return bRumbleFade;
}

/**
Method for activating fading
\param activate true for activating fading else false
*/
void Rumble::ActivateFade(bool activate)
{
	bRumbleFade = activate;
}

/**
Getter for the effect difference
used for fading
\return effect difference
*/
double Rumble::GetEffectDifference()
{
	return rumbleEffectDifference;
}

/**
Getter for the old intensity value
\return old intensity value
*/
double Rumble::GetOldIntensityValue()
{
	return rumbleOldIntensityValue;
}

/**
Getter for the effect duration
\return duration
*/
double Rumble::GetDuration()
{
	return rumbleDuration;
}

/**
Getter for the intensity
\return intensity
*/
double Rumble::GetIntensity()
{
	return rumbleIntensity;
}

/**
Getter for the fading time
\return fading time
*/
double Rumble::GetFadeTime()
{
	return rumbleFadeTime;
}

/**
Setter for the intensity
\param intensity
*/
void Rumble::SetIntensity(double intensity)
{
	rumbleIntensity = intensity;
}

/**
Setter for the duration
\param duration
*/
void Rumble::SetDuration(double duration)
{
	rumbleDuration = duration;
}

/**
Setter for the old intensity
\param old intensity value
*/
void Rumble::SetOldIntensityValue(double oldIntensity)
{
	rumbleOldIntensityValue = oldIntensity;
}

/**
Setter for the effect difference
\param effect difference
*/
void Rumble::SetEffectDifference(double difference)
{
	rumbleEffectDifference = difference;
}

/**
Setter for the fade time
\param fade time
*/
void Rumble::SetFadeTime(double fadeTime)
{
	rumbleFadeTime = fadeTime;
}

/**
Getter for retrieving the rumble
\return amBX rumble
*/
IamBX_Rumble *Rumble::GetRumble()
{
	return m_rumble;
}
#endif
