/*
 * This file is part of SEMP.
 *
 * SEMP is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEMP is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SEMP. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2009, 2010, 2011, 2012 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */

#include "RumbleContainer.h"

#if __USEAMBX__
/**
Constructor
*/
RumbleContainer::RumbleContainer()
{
}

/**
Destructor
*/
RumbleContainer::~RumbleContainer()
{
}

/**
Method for adding a rumble to a location
\param location for the rumble to be added to
\param rumble with the rumble to be added
*/
void RumbleContainer::AddRumble(amBX_loc location, Rumble* rumble)
{
	rumbleMap.insert(std::make_pair(location, rumble));	  
}

/**
Method for retrieving a rumble
\param location from which the rumble should be retrieved
\return rumble
*/
Rumble* RumbleContainer::GetRumble(amBX_loc location)
{
	std::map<amBX_loc, Rumble*>::iterator iter = rumbleMap.find(location);
	if(iter == rumbleMap.end())
		return NULL;
	else
		return iter->second;
}

/**
Method for retrieving all rumbles
\return vector with all rumbles
*/
std::vector<Rumble *> RumbleContainer::GetAllRumbles()
{
	std::vector<Rumble *> retVal;

	for( std::map<amBX_loc, Rumble*>::iterator it = rumbleMap.begin(); it != rumbleMap.end(); ++it ) {
        retVal.push_back(it->second);
    }

	return retVal;
}
#endif
