/*
 * This file is part of SEMP.
 *
 * SEMP is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEMP is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SEMP. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2009, 2010, 2011, 2012 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */

/**
This file describes the storage structure for Scent effects
*/

#include "Scent.h"

/**
Constructor
\param id fan id of the scent device
\param scent string with the scent identification
*/
Scent::Scent(int id, std::string scent)
{
	fanId = id;
	bScentFade = false;
	scentFadeTime = -1.0;
	scentEffectDifference = -1.0;
	scentDuration = -1.0;
	scentIntensity = -1.0;
	scentOldIntensityValue = 0.0;
	scentPTS = -1.0;
	scentType = scent;
}

/**
Destructor
*/
Scent::~Scent()
{
}

/**
Getter for the fan id
\return fan id
*/
int Scent::GetFanID()
{
	return fanId;
}

/**
Method for resetting hte scent
*/
void Scent::ResetScent()
{
	bScentFade = false;
	scentFadeTime = -1.0;
	scentDuration = -1.0;
	scentIntensity = -1.0;
	scentOldIntensityValue = 0.0;
	scentEffectDifference = -1.0;
	scentPTS = -1.0;
}

/**
Method for retrieving information about
the fan fade activity
\return true if fan fade is active else false
*/
bool Scent::IsFadeActive()
{
	return bScentFade;
}

/**
Getter for the timestamp
\return timestamp
*/
double Scent::GetPTS()
{
	return scentPTS;
}

/**
Setter for the timestamp
\param pts with the timestamp
*/
void Scent::SetPTS(double pts)
{
	scentPTS = pts;
}

/**
Method for activating fading
\param activate true for activating else false
*/
void Scent::ActivateFade(bool activate)
{
	bScentFade = activate;
}

/**
Getter for the duration attribute
\return duration
*/
double Scent::GetDuration()
{
	return scentDuration;
}

/**
Getter for the intensity attribute
\return intensity
*/
double Scent::GetIntensity()
{
	return scentIntensity;
}

/**
Getter for the fade attribute
\return fade time
*/
double Scent::GetFadeTime()
{
	return scentFadeTime;
}

/**
Setter for the intensity
\param intensity
*/
void Scent::SetIntensity(double intensity)
{
	scentIntensity = intensity;
}

/**
Setter for the duration
\param duration
*/
void Scent::SetDuration(double duration)
{
	scentDuration = duration;
}

/**
Setter for the fading time
\param fadeTime with the fading time
*/
void Scent::SetFadeTime(double fadeTime)
{
	scentFadeTime = fadeTime;
}

/**
Getter for the scent identifier
\return string with the identifier
*/
std::string Scent::GetScent()
{
	return scentType;
}

/**
Setter for the difference between two effects
only needed for fading
\param difference between effects
*/
void Scent::SetEffectDifference(double difference)
{
	scentEffectDifference = difference;
}

/**
Getter for the effect difference
\return difference between effects
*/
double Scent::GetEffectDifference()
{
	return scentEffectDifference;
}

/**
Setter for storing the old intensity
\param old intensity 
*/
void Scent::SetOldIntensityValue(double oldIntensity)
{
	scentOldIntensityValue = oldIntensity;
}

/**
Getter for the old intensity value
\return old intensity
*/
double Scent::GetOldIntensityValue()
{
	return scentOldIntensityValue;
}