/*
 * This file is part of SEMP.
 *
 * SEMP is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEMP is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SEMP. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2009, 2010, 2011, 2012 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */

#include "amBXErrorHandler.h"

#if __USEAMBX__
/**
Constructor
*/
amBXErrorHandler::amBXErrorHandler()
{
}

/**
Destructor
*/
amBXErrorHandler::~amBXErrorHandler()
{
}

/**
Method for translating an error message
\param amBX_RESULT error message
\return string with the error message
*/
std::string amBXErrorHandler::GetErrorMessage(amBX_RESULT errorNumber)
{
	std::string msg = "";
	switch (errorNumber)
	{
	case amBX_OK:
		msg = "OK";
		break;
	case amBX_NO_SPACE:
		msg = "out of buffer space";
		break;
	case amBX_ERROR:
		msg = "generic error";
		break;
	case amBX_NOT_FOUND:
		msg = "no file/device etc. found";
		break;
	case amBX_VERSION_NOT_FOUND:
		msg = "The requested version of amBX API is not available";
		break;
	case amBX_BAD_ARGS:
		msg = "arguments passed in bad (usually null pointers passed in)";
		break;
	case amBX_OUT_OF_RANGE:
		msg = "Value given is beyond usable range using default values if possible";
		break;
	case amBX_OUT_OF_MEMORY:
		msg = "couldn't allocate memory";
		break;
	case amBX_NOT_INSTALLED:
		msg = "amBX isn't installed on the PC! Continue without amBX support";
		break;
	case amBX_OLD_VERSION:
		msg = "amBX installed, but need newer version";
		break;
	case amBX_ENGINE_LOST:
		msg = "connection to amBX Engine currently inoperative";
		break;
	case amBX_SENDING_TIMEOUT:
		msg = "request to send a script timed out";
		break;
	case amBX_NOT_THREADED:
		msg = "returned by a threaded function if threading is no being used for the purpose specified";
		break;
	case amBX_BAD_THREADID:
		msg = "returned by a function if the thread ID is incorrect or doesn't exist. The thread with the ID could have been recently removed";
		break;
	case amBX_THREAD_EXISTS:
		msg = "returned by a threaded function if a thread is currently being used for that function";
		break;
	case amBX_UPDATE_TIMEOUT:
		msg = "returned if waiting to perform an update times out. This could be waiting for a thread to finish or waiting for a critical section";
		break;
	case amBX_THREAD_TIMEOUT:
		msg = "returned if the request to run a thread times out. This could be because another thread is already being run or another thread is using the resources needed to run the thread";
		break;
	default:
		msg = "Unknown error";
	}

	return msg;
}
#endif
